#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <math.h>
#include <stdbool.h>
#include <unistd.h>
#include <termios.h>
#include <limits.h>
#include <time.h>
#include <sys/time.h>
#include <SDL2/SDL.h>
#include <SDL2/SDL_image.h>

#include "structure.h"
#include "cars.h"
#include "base.h"
#include "collisions.h"

// use after updating the position
void updateChunk(int nPl) {
    car* c = players[nPl].c;
    if(c->pos.fx < 0.0) {
        c->pos.fx += 1.0*ROOM_SIZE;
        c->chx -= 1;
    }
    if(c->pos.fx >= 1.0*ROOM_SIZE) {
        c->pos.fx -= 1.0*ROOM_SIZE;
        c->chx += 1;
    }
    if(c->pos.fy < 0.0) {
        c->pos.fy += 1.0*ROOM_SIZE;
        c->chy -= 1;
    }
    if(c->pos.fy >= 1.0*ROOM_SIZE) {
        c->pos.fy -= 1.0*ROOM_SIZE;
        c->chy += 1;
    }
}

ptf get_absolute_coords(int nPl) {
    return (ptf){
        .fx = players[nPl].c->pos.fx + ROOM_SIZE*players[nPl].c->chx,
        .fy = players[nPl].c->pos.fy + ROOM_SIZE*players[nPl].c->chy
    };
}

// -------------------------------------------------------------------------------- //
// make the player move DT (does not take friction into consideration)

// start and finish
void move_on_ENDS(int nPl) {
    car* c = players[nPl].c;
    c->pos.fx += c->vel.fx*DT;
    c->pos.fy += c->vel.fy*DT;
    if(
        (START_DIR != 3 && c->pos.fx-PLAYER_R/2 <= 0.0 && c->vel.fx < 0.0) ||
        (START_DIR == 3 && c->pos.fx-PLAYER_R/2 <= 0.0 && c->vel.fx < 0.0 && (c->pos.fy-PLAYER_R/2 <= ROOM_SIZE*DIST_EDGE || c->pos.fy+PLAYER_R/2 >= ROOM_SIZE*(1.0-DIST_EDGE)))
    ) {         // left wall
        c->vel.fx *= -1.0;
        c->pos.fx += 2.0*c->vel.fx*DT;
        c->vel.fx *= RESTITUTION_WALL;
    }
    if(
        (START_DIR != 1 && c->pos.fx+PLAYER_R/2 >= 1.0*ROOM_SIZE && c->vel.fx > 0.0) ||
        (START_DIR == 1 && c->pos.fx+PLAYER_R/2 >= 1.0*ROOM_SIZE && c->vel.fx > 0.0 && (c->pos.fy-PLAYER_R/2 <= ROOM_SIZE*DIST_EDGE || c->pos.fy+PLAYER_R/2 >= ROOM_SIZE*(1.0-DIST_EDGE)))
    ) {         // right wall
        c->vel.fx *= -1.0;
        c->pos.fx += 2.0*c->vel.fx*DT;
        c->vel.fx *= RESTITUTION_WALL;
    }
    if(
        (START_DIR != 0 && c->pos.fy-PLAYER_R/2 <= 0.0 && c->vel.fy < 0.0) ||
        (START_DIR == 0 && c->pos.fy-PLAYER_R/2 <= 0.0 && c->vel.fy < 0.0 && (c->pos.fx-PLAYER_R/2 <= ROOM_SIZE*DIST_EDGE || c->pos.fx+PLAYER_R/2 >= ROOM_SIZE*(1.0-DIST_EDGE)))
    ) {         // top wall
        c->vel.fy *= -1.0;
        c->pos.fy += 2.0*c->vel.fy*DT;
        c->vel.fy *= RESTITUTION_WALL;
    }
    if(
        (START_DIR != 2 && c->pos.fy+PLAYER_R/2 >= 1.0*ROOM_SIZE && c->vel.fy > 0.0) ||
        (START_DIR == 2 && c->pos.fy+PLAYER_R/2 >= 1.0*ROOM_SIZE && c->vel.fy > 0.0 && (c->pos.fx-PLAYER_R/2 <= ROOM_SIZE*DIST_EDGE || c->pos.fx+PLAYER_R/2 >= ROOM_SIZE*(1.0-DIST_EDGE)))
    ) {         // bottom wall
        c->vel.fy *= -1.0;
        c->pos.fy += 2.0*c->vel.fy*DT;
        c->vel.fy *= RESTITUTION_WALL;
    }
}

void move_on_STR_V(int nPl) {
    car* c = players[nPl].c;
    c->pos.fx += c->vel.fx*DT;
    c->pos.fy += c->vel.fy*DT;
    if(c->pos.fx-PLAYER_R/2 <= ROOM_SIZE*DIST_EDGE && c->vel.fx < 0.0) {         // left wall
        c->vel.fx *= -1.0;
        c->pos.fx += 2.0*c->vel.fx*DT;
        c->vel.fx *= RESTITUTION_WALL;
    }
    if(c->pos.fx+PLAYER_R/2 >= ROOM_SIZE*(1.0-DIST_EDGE) && c->vel.fx > 0.0) {     // right wall
        c->vel.fx *= -1.0;
        c->pos.fx += 2.0*c->vel.fx*DT;
        c->vel.fx *= RESTITUTION_WALL;
    }
}

void move_on_STR_H(int nPl) {
    car* c = players[nPl].c;
    c->pos.fx += c->vel.fx*DT;
    c->pos.fy += c->vel.fy*DT;
    if(c->pos.fy-PLAYER_R/2 <= ROOM_SIZE*DIST_EDGE && c->vel.fy < 0.0) {         // top wall
        c->vel.fy *= -1.0;
        c->pos.fy += 2.0*c->vel.fy*DT;
        c->vel.fy *= RESTITUTION_WALL;
    }
    if(c->pos.fy+PLAYER_R/2 >= ROOM_SIZE*(1.0-DIST_EDGE) && c->vel.fy > 0.0) {     // bottom wall
        c->vel.fy *= -1.0;
        c->pos.fy += 2.0*c->vel.fy*DT;
        c->vel.fy *= RESTITUTION_WALL;
    }
}

void move_on_TURN(int nPl, int cenX, int cenY) {
    car* c = players[nPl].c;
    ptf prev = (ptf){.fx = c->pos.fx, .fy = c->pos.fy};
    c->pos.fx += c->vel.fx*DT;
    c->pos.fy += c->vel.fy*DT;
    ptf next = c->pos;

    double prevDist = distance(prev, (ptf){.fx = 1.0*cenX, .fy = 1.0*cenY});
    double nextDist = distance(next, (ptf){.fx = 1.0*cenX, .fy = 1.0*cenY});

    // inner circle
    if(prevDist-PLAYER_R/2 > ROOM_SIZE*DIST_EDGE && nextDist-PLAYER_R/2 <= ROOM_SIZE*DIST_EDGE) {
        ptf to_in = normalize((ptf){
            .fx = cenX-prev.fx,
            .fy = cenY-prev.fy
        });
        double inwards = dot(to_in, c->vel);
        c->vel = add(c->vel, (ptf){.fx = -2.0*to_in.fx*inwards, .fy = -2.0*to_in.fy*inwards});

        c->pos.fx += 2.0*c->vel.fx*DT;
        c->pos.fy += 2.0*c->vel.fy*DT;
    }

    // outer circle
    if(prevDist+PLAYER_R/2 < ROOM_SIZE*(1.0-DIST_EDGE) && nextDist+PLAYER_R/2 >= ROOM_SIZE*(1.0-DIST_EDGE)) {
        ptf to_out = normalize((ptf){
            .fx = prev.fx-cenX,
            .fy = prev.fy-cenY
        });
        double outwards = dot(to_out, c->vel);
        c->vel = add(c->vel, (ptf){.fx = -2.0*to_out.fx*outwards, .fy = -2.0*to_out.fy*outwards});

        c->pos.fx += 2.0*c->vel.fx*DT;
        c->pos.fy += 2.0*c->vel.fy*DT;
    }
}

void bumpOtherCars(int nPl) {
    for(int p = 0; p < nPlayers; p++) {
        if(p != nPl) {
            double sep = distance(
                get_absolute_coords(p),
                get_absolute_coords(nPl)
            );
            if(sep <= PLAYER_R) {
                // p_total is multiplied by RESTITUTION_PLAYER
                ptf director = normalize((ptf){
                    .fx = players[p].c->pos.fx - players[nPl].c->pos.fx,
                    .fy = players[p].c->pos.fy - players[nPl].c->pos.fy
                });
                double speed_P = norm(players[p].c->vel);
                double speed_N = norm(players[nPl].c->vel);

                players[p].c->vel.fx = director.fx*RESTITUTION_PLAYER*(speed_N);
                players[p].c->vel.fy = director.fy*RESTITUTION_PLAYER*(speed_N);

                players[nPl].c->vel.fx = -director.fx*RESTITUTION_PLAYER*(speed_P);
                players[nPl].c->vel.fy = -director.fy*RESTITUTION_PLAYER*(speed_P);
            }
        }
    }
}

void apply_friction(int nPl) {
    players[nPl].c->vel.fx *= (1.0 - FRICTION*DT);
    players[nPl].c->vel.fy *= (1.0 - FRICTION*DT);
}

// -- //

// return true iif the car has moved
bool updateCar(level* lvl, int nPl) {
    if(norm(players[nPl].c->vel) >= DV) {
        switch (lvl->map[players[nPl].c->chy][players[nPl].c->chx]){
            case NONE:
                fprintf(stderr, "[player %d] Is in an empty chunk (at %d %d)\n", nPl, players[nPl].c->chx, players[nPl].c->chy);
                return false;

            case START:
            case END:
                move_on_ENDS(nPl);
                bumpOtherCars(nPl);
                apply_friction(nPl);
                updateChunk(nPl);
                return true;
                
            case STR_V:
                move_on_STR_V(nPl);
                bumpOtherCars(nPl);
                apply_friction(nPl);
                updateChunk(nPl);
                return true;

            case STR_H:
                move_on_STR_H(nPl);
                bumpOtherCars(nPl);
                apply_friction(nPl);
                updateChunk(nPl);
                return true;

            case TURN_NE:
                move_on_TURN(nPl, ROOM_SIZE, 0);
                bumpOtherCars(nPl);
                apply_friction(nPl);
                updateChunk(nPl);
                return true;

            case TURN_NW:
                move_on_TURN(nPl, 0, 0);
                bumpOtherCars(nPl);
                apply_friction(nPl);
                updateChunk(nPl);
                return true;

            case TURN_SE:
                move_on_TURN(nPl, ROOM_SIZE, ROOM_SIZE);
                bumpOtherCars(nPl);
                apply_friction(nPl);
                updateChunk(nPl);
                return true;

            case TURN_SW:
                move_on_TURN(nPl, 0, ROOM_SIZE);
                bumpOtherCars(nPl);
                apply_friction(nPl);
                updateChunk(nPl);
                return true;
            
            default:
                return false;
        }
    } else {
        return false;
    }
}

// return true if at least one car moved
bool updateCars(level* lvl) {
    bool res = false;
    for(int p = 0; p < nPlayers; p++) {
        bool rr = updateCar(lvl, p);
        res = res || rr;
    }
    return res;
}