#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <math.h>
#include <stdbool.h>
#include <unistd.h>
#include <string.h>
#include <limits.h>
#include <time.h>
#include <sys/time.h>
#include <SDL2/SDL.h>
#include <SDL2/SDL_image.h>

#include "structure.h"
#include "base.h"
#include "display.h"
#include "collisions.h"
#include "rooms.h"
#include "cars.h"

void cameraActions(level* lvl, bool* halt, int* cx, int* cy, int* dezoom, int* sizeR) {
    SDL_Event event;
    while(SDL_PollEvent(&event)) {
        switch (event.type) {
            case SDL_QUIT:
                break;
            case SDL_KEYDOWN:
                switch (event.key.keysym.sym) {
                    case SDLK_SPACE:
                        *halt = true;
                        break;

                    case SDLK_q:
                        *cy = max(0, *cy-1);
                        break;

                    case SDLK_z:
                        *cx = max(0, *cx-1);
                        break;

                    case SDLK_d:
                        *cy = min(lvl->lines-1, *cy+1);
                        break;

                    case SDLK_s:
                        *cx = min(lvl->cols-1, *cx+1);
                        break;

                    case SDLK_p:
                        *dezoom = max(1, *dezoom-1);
                        *sizeR = min(WIDTH, HEIGHT)/(2*(*dezoom)+1)+1;
                        break;

                    case SDLK_m:
                        *dezoom = min(3, *dezoom+1);
                        *sizeR = min(WIDTH, HEIGHT)/(2*(*dezoom)+1)+1;
                        break;

                    default:
                        break;
                }
        }
    }
}

void player_turn(int id, char* exec) {
    system(exec);
    FILE* ptr = fopen("bots/answer.txt", "r");
    int angle, mult;
    fscanf(ptr, "%d %d", &angle, &mult);
    fclose(ptr);
    set_speed_car(players[id].c, MAX_SPEED*mult/100.0*cos(angle*3.14159/180.0), MAX_SPEED*mult/100.0*sin(angle*3.14159/180.0));
}

void parse_all_players(level* lvl, char** execs) {
    currentTurn += 1;
    for(int p = 0; p < nPlayers; p++) {
        write_output("output.txt", lvl, p);
        usleep(1000000/10);
        player_turn(p, execs[p]);
    }
}

int main(int argc, char** argv) {
    if(argc < 2) {
        fprintf(stderr, "usage : ./back <level_path> <P1 executable> <P2 executable> ...\n");
        exit(1);
    }
    int N_PLAYERS = argc-2;

    char** execs = malloc(sizeof(char*)*N_PLAYERS);
    for(int k = 0; k < N_PLAYERS; k++) {
        execs[k] = malloc(sizeof(char)*64);
        execs[k][0] = '.';
        execs[k][1] = '/';
        int i = 2;
        while(i-2 < 64 && argv[k+2][i-2] != '\0') {
            execs[k][i] = argv[k+2][i-2];
            i += 1;
        }
        execs[k][i] = '\0';
    }

    for(int k = 0; k < N_PLAYERS; k++) {
        printf("[%d] %s\n", k, execs[k]);
    }

    double elapsed = 0.0;
    bool halt = false;
    srand(time(NULL));
    if (SDL_Init(SDL_INIT_EVERYTHING) != 0) {
        printf("error initializing SDL: %s\n", SDL_GetError());
    }
    SDL_Window* win = SDL_CreateWindow("Game",
                                       SDL_WINDOWPOS_CENTERED,
                                       SDL_WINDOWPOS_CENTERED,
                                       WIDTH, HEIGHT, 0);
 
    Uint32 render_flags = SDL_RENDERER_ACCELERATED;
    SDL_Renderer* rend = SDL_CreateRenderer(win, -1, render_flags);
    SDL_SetRenderDrawBlendMode(rend, SDL_BLENDMODE_BLEND);

    import_letters(rend);
    import_digits(rend);
    // ---------------------------- //
    //stage_file("levels/stage.txt");
    level* stage = parse_level(argv[1]);
    init_all(N_PLAYERS);

    printf("Starting direction : ");
    if(START_DIR == 0) {
        printf("North\n");
    } else if(START_DIR == 1) {
        printf("East\n");
    } else if(START_DIR == 2) {
        printf("South\n");
    } else if(START_DIR == 3) {
        printf("West\n");
    } else {
        printf("???\n");
    }

    int cx = 1;
    int cy = 2;
    int dezoom = 2;
    int sizeR = 200;
    while(!halt) {
        resetRenderer(rend);
        renderMap(rend, stage, cx, cy, dezoom, sizeR);
        for(int p = 0; p < N_PLAYERS; p++) {
            pt proj = project(p, cx, cy, dezoom, sizeR);
            drawStringToRenderer(rend, &(execs[p][1]), proj.ix, proj.iy, 75/(4+dezoom), 105/(4+dezoom));
        }
        if(elapsed <= 0.7) {
            // just selected speeds
            for(int p = 0; p < N_PLAYERS; p++) {
                pt proj = project(p, cx, cy, dezoom, sizeR);
                drawLineWithThicc(rend, 2,
                    proj.ix,
                    proj.ix+(int)(players[p].c->vel.fx/5.0),
                    proj.iy,
                    proj.iy+(int)(players[p].c->vel.fy/5.0),
                    players[p].rgb.red, players[p].rgb.green, players[p].rgb.blue, 255
                );
            }
            placeRectToRenderer(rend, 0, 0, 50, 50, 255, 255, 32, 192);
        } else if(updateCars(stage)) {
            // movement
            placeRectToRenderer(rend, 0, 0, 50, 50, 32, 255, 32, 192);
        } else {
            // no movement
            placeRectToRenderer(rend, 0, 0, 50, 50, 255, 32, 32, 192);
            parse_all_players(stage, execs);
            elapsed = 0.0;
        }
        draw7SegNumberToRenderer(rend, currentTurn, WIDTH/2, 30, 25, 25, 3, 255, 255, 255, 255, 0, 0);
        updateRenderer(rend);
        elapsed += DT;
        usleep((int)(1000000*DT));
        cameraActions(stage, &halt, &cx, &cy, &dezoom, &sizeR);
    }
    
    // ---------------------------- //
    free_digits(digits);
    free_digits(letters);
    free_level(stage);
    for(int k = 0; k < N_PLAYERS; k++) {
        free(execs[k]);
    }
    free(execs);

    SDL_DestroyRenderer(rend);
    SDL_DestroyWindow(win);
    SDL_Quit();
    return 0;
}

/* TODO :
    - add level generation
    - add player input parsing (and some dumb code to stage it)
    - add function that gives players the necessary data to play
    - check seg faults
    + add items
    + add hazards
*/