#include <stdio.h>
#include <SDL2/SDL.h>
#include <SDL2/SDL_image.h>
#include "structure.h"

const int WIDTH = 1000;
const int HEIGHT = 1000;

const int ROOM_SIZE = 100;
const int MAX_SPEED = 200;               // m/s

const double DIST_EDGE = 0.2;
const double START_EDGE = 0.05;
const double RESTITUTION_WALL = 0.8;
const double RESTITUTION_PLAYER = 0.8;  // keep equal to prevouis constant

const int PLAYER_R = 10;
const int MAX_THETA_SPAWN = 80;         // degrees

const int BARRIERS = 1;
const double BARRIER_WIDTH = 0.05;

const double FRICTION = 0.90;
const double DV = 2.2;                  // m/s
const double DT = 1.0/100.0;
const double EPSILON = 1.0/4096.0;

const double DELTA_V = MAX_SPEED/30;    // m/s
const double DELTA_THETA = 5;           // degrees

// ------------------------------------------------------------------------------------------ //

void write_output(char* stream, level* lvl, int nPl) {
    FILE* ptr = fopen(stream, "w");
    fprintf(ptr, "%d\n",nPlayers);
    for(int p = 0; p < nPlayers; p++) {
        fprintf(ptr, "%d (%d %d) (%.2lf %.2lf)\n", p, players[p].c->chx, players[p].c->chy, players[p].c->pos.fx, players[p].c->pos.fy);
    }
    fprintf(ptr, "[%d]\n\n", nPl);
    fprintf(ptr, "%d %d\n", lvl->lines, lvl->cols);
    for(int l = 0; l < lvl->lines; l++) {
        for(int c = 0; c < lvl->cols; c++) {
            switch (lvl->map[l][c]){
                case NONE:
                    fprintf(ptr, ".");
                    break;

                case START:
                    fprintf(ptr, "S");
                    break;

                case END:
                    fprintf(ptr, "E");
                    break;
                
                default:
                    fprintf(ptr, "%d", (int)lvl->map[l][c]-3);
                    break;
            }
        }
        fprintf(ptr, "\n");
    }
    fprintf(ptr, "\n");
    fprintf(ptr, "%d\n", ROOM_SIZE);
    fprintf(ptr, "%d\n", MAX_SPEED);
    fprintf(ptr, "%d\n", PLAYER_R);
    fprintf(ptr, "%.2lf\n", FRICTION);
    fprintf(ptr, "%.2lf\n", RESTITUTION_WALL);
    fprintf(ptr, "%.2lf\n", DIST_EDGE);
    fprintf(ptr, "\n\nRoom Size\nMaximum speed\nPlayer Radius\nFriction\nRestitution factor\nTrack Distance to Edge\n");
    fclose(ptr);
}